﻿using System.Collections.Generic;
using System.Linq;
using System.Web.Mvc;
using Tradukisto.Engine.ResourceProviders;

namespace Tradukisto.Engine.DataAnnotations
{
    public class LocalizableModelValidator : ModelValidator
    {
        private ModelValidator _innerValidator;
        private ModelMetadata _metadata;

        public LocalizableModelValidator(ModelValidator innerValidator, ModelMetadata metadata, ControllerContext controllerContext)
            : base(metadata, controllerContext)
        {
            _innerValidator = innerValidator;
            _metadata = metadata;
        }

        public override IEnumerable<ModelValidationResult> Validate(object container)
        {
            // execute the inner validation which doesn't have localization
            var results = _innerValidator.Validate(container);
            // convert the error message (which should be the localization resource key) to the localized value through the ILocalizationResourceProvider
            var source = LocalizationProvider.Current;
            return results.Select((result) =>
            {
                var key = result.Message;
                var message = source.GetString(key);
                return new ModelValidationResult() { Message = string.Format(message, _metadata.DisplayName) };
            });
        }
    }
}