﻿using System;

namespace Tradukisto.Engine.ResourceProviders
{
    public class LocalizationProvider
    {
        private static ILocalizationProvider _instance;
        private static Func<ILocalizationProvider> _initializer;
        private static Func<string> _cultureNameResolver;
        private static object _aLock;

        public static ILocalizationProvider Current
        {
            get
            {
                if (_instance == null)
                {
                    lock (_aLock)
                    {
                        if (_instance == null)
                        {
                            if (_initializer == null)
                            {
                                throw new ArgumentNullException("_initializer");
                            }
                            _instance = _initializer.Invoke();
                        }
                    }
                }
                return _instance;
            }
        }

        public static string CultureName
        {
            get
            {
                return _cultureNameResolver.Invoke();
            }
        }

        static LocalizationProvider()
        {
            _aLock = new object();
        }

        private LocalizationProvider()
        {
        }

        internal static void RegisterCultureNameResolver(Func<string> cultureNameResolver)
        {
            _cultureNameResolver = cultureNameResolver;
        }

        internal static void RegisterResourceProvider(Func<ILocalizationProvider> initializer)
        {
            _initializer = initializer;
        }
    }
}
