﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.Mvc;

namespace Tradukisto.Engine.ResourceProviders
{
    public abstract class LocalizationProviderBase : ILocalizationProvider
    {
        public Type ResourceType { get; set; }

        protected LocalizationProviderBase()
        {
        }

        public string GetString(string cultureName, string key)
        {
            return OnGetString(cultureName, key.ToLower());
        }

        public string GetString(string key)
        {
            // find the localized result only if the key is not null
            var result = string.IsNullOrWhiteSpace(key) ? null : GetString(LocalizationProvider.CultureName, key);
            // return the original key if didn't find the localized result
            return string.IsNullOrWhiteSpace(result) ? key : result;
        }

        public IHtmlString GetHtmlString(string cultureName, string key)
        {
            return MvcHtmlString.Create(GetString(cultureName, key.ToLower()));
        }

        public IHtmlString GetHtmlString(string key)
        {
            return MvcHtmlString.Create(GetString(key));
        }

        protected abstract string OnGetString(string cultureName, string key);

        public abstract Dictionary<string, string> GetAllStringsByControllerAndAction(string culture, string controller, string action);

        public abstract Dictionary<string, string> GetAllStringsByControllerAndAction(string controller, string action);

    }
}
